/**
 * Controller for profiles.
 *
 * @param {XhrService} Service for performing XHR requests.
 *
 * @author Elliott Sprehn (esprehn@gmail.com)
 * @constructor
 */
function ProfileController(xhr) {
  this.xhr = xhr;
  this.friends = [];
  this.feed = [];
}

/**
 * Loads specific friends.
 *
 * @param {string} opt_filter Filter to control which friends are loaded.
 * @param {Function} callback Callback called with the mathcing friends.
 */
ProfileController.prototype.loadFriends = function(opt_filter, callback) {  
  var query = this.convertToQuery_(this.parseFilter_(opt_filter || ''));
  this.xhr.get('/friends' + query, function(friends) {
    this.friends = friends;
    (callback || _.identity)(friends);
  }, this);
};

/**
 * Finds friends that match. 
 *
 * @param {Function} callback Callback called with the mathcing friends.
 */
ProfileController.prototype.loadFeed = function(callback) {
  var friendIds = _(this.friends).pluck('id').join(',');
  this.xhr.get('/feed?friends=' + friendIds, function(feed) {
    this.feed = feed;
    (callback || _.identity)(feed);
  }, this);
};

/**
 * Converts a filter string with possible keyword params into an object.
 *
 * @param {string} opt_filter Filter to convert.
 * @return {Object.<string, string>} Filter map.
 * @private
 */
ProfileController.prototype.parseFilter_ = function(filter) {
  var result = {
    search: ''
  };
  var values = filter.split(' ');
  for (var i = 0; i < values.length; ++i) {
    var parts = values[i].split(':');
    if (parts.length > 1) {
      var key = parts.shift();
      var value = parts.join(':');
      if (value.match(/^"/)) {
        value = value.replace(/^"/, '');
        for (i += 1; i < values.length; ++i) {
          value += ' ' + values[i];
          if (values[i].match(/"$/)) break;
        }
        value = value.replace(/"$/, '');
      }
      result[key.toLowerCase()] = value;
    } else {
      result.search = $.trim(result.search + ' ' + values[i]);
    }
  }
  return result;
};

/**
 * Converts a map into a query string.
 *
 * @param {Object.<string, string>} map Map to convert.
 * @return {string} query string.
 * @private
 */
ProfileController.prototype.convertToQuery_ = function(map) {
  var query = '';
  _(map).each(function(value, key) {
    if (!value.length) return;
    query += (query.length ? '&' : '?');
    query += encodeURIComponent(key) + '=' + encodeURIComponent(value);
  });
  return query;
};
