/**
 * Profiles view.
 *
 * @param {jQuery} context jQuery wrapped node to use as the view.
 * @param {ProfileController} Profile controller.
 *
 * @author Elliott Sprehn (esprehn@gmail.com)
 * @constructor
 */
function ProfileView(context, controller) {

  context.find('#friends input[name=filter]').bind('keyup change', function(e) {
    var value = e.target.value;
    context.find('#friends button[name=reset]').attr('disabled', !value.length);
    update(value);
  });

  context.find('#friends button[name=reset]').click(function() {
    setFilter('');
  });

  /**
   * Load the view using the default filter in the input.
   */
  this.load = function() {
    context.find('#friends input[name=filter]').trigger('change');
  };

  /**
   * Update the whole view, loading a new set of friends and their posts.
   * Optionally filtered by a search query.
   *
   * @param {string} opt_filter Optional search query.
   */
  function update(opt_filter) {
    controller.loadFriends(opt_filter || '', function(friends) {
      updateFriends(friends);
      updateFeed([]);
      controller.loadFeed(function(feed) {
        updateFeed(friends, feed);
      });
    });
  }

  /**
   * Update the listing of friends.
   *
   * @param {Array.<Object>} friends Friends to map to posts.
   */
  function updateFriends(friends) {
    var input = context.find('#friends input');
    var list = context.find('#friends ul');
    list.html('');
    _(friends).each(function(friend) {
      var li = $('<li><img><span></span>');
      li.find('img').attr('src', 'photos/' + friend.id + '.jpeg');
      li.find('span').text(friend.name);
      li.click(function() {
        setFilter('name:"' + friend.name + '"');
      });
      list.append(li);
    });
  }

  /**
   * Update the listing of posts.
   *
   * @param {Array.<Object>} friends Friends to map to posts.
   * @param {Array.<Object>} posts Posts to display.
   */
  function updateFeed(friends, posts) {
    var feed = context.find('#feed ul');
    feed.html('');
    _(posts).each(function(post) {
      var item = $(
        '<li>' +
        ' <h3>' +
        '  <img>' +
        '  <span class="name"></span> at ' +
        '  <span class="time"></span>' +
        ' </h3>' +
        ' <p></p>' +
        '</li>'
      );
      item.find('img').attr('src', 'photos/' + post.user + '.jpeg');
      item.find('.name').text(_(friends).find(function(friend) {
        return friend.id === post.user;
      }).name);
      item.find('.time').text(post.time);
      item.find('p').text(post.text);
      feed.append(item);
    });
  }

  /**
   * Set the value of the filter input and trigger an update.
   *
   * @param {string} value Value to assign to the filter input.
   */
  function setFilter(value) {
    context.find('#friends input[name=filter]').val(value).trigger('change');
  }
}
